<?php

namespace EventPortal\Domain\Event\Models;

use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Concerns\HasStatus;
use EventPortal\Concerns\HasTeams;
use EventPortal\Contracts\Statusable;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use Iben\Statable\Statable;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class Event extends Model implements AuditableInterface, Statusable
{
    use HasTable,
        AddToFillable,
        AddToCasts,
        Auditable,
        Statable,
        HasStatus,
        HasTeams;

    public const KEY = 'event';

    /** @var array<string> */
    protected $fillable = [
        'name',
        'url',
        'status',
    ];

    /** @var array<string> */
    protected $with = [
        'teams',
    ];

    protected function getGraph(): string
    {
        return 'event';
    }

    public function delegates(): BelongsToMany
    {
        return $this
            ->belongsToMany(Delegate::class)
            ->using(EventDelegate::class)
            ->withPivot(['id', 'status'])
            ->as('delegates')
            ->withTimestamps();
    }

    public function transitionRouteName(): string
    {
        return 'admin.event.transition';
    }

    public function transitionRouteParameters(): array
    {
        return [
            'event' => $this->id,
        ];
    }
}
