<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Inertia;
use Inertia\Response;

class UserController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:user.show')->only('show');
        $this->middleware('check-permission:user.store')->only(['create', 'store']);
        $this->middleware('check-permission:user.update')->only(['edit', 'update']);
        $this->middleware('check-permission:user.destroy')->only('delete');
    }

    public function index(): Response
    {
        $query = Admin::query()
            ->join('users', function (JoinClause $query) {
                return $query->on('admins.user_id', '=', 'users.id');
            });

        $table = Table::make('admin.index')
            ->query($query)
            ->request(request())
            ->get();

        return Inertia::render('User/Index', [
            'data' => $table,
        ]);
    }

    public function create(): Response
    {
        $form = Form::make('admin.create')->model(new Admin)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Users', route('admin.user.index'))
            ->add('Create a user')
            ->get();

        return Inertia::render('User/Create', [
            'breadcrumbs' => $breadcrumbs,
            'form' => $form,
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        Form::make('admin.create')
            ->model(new Admin)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.user.index');
    }

    public function edit(Admin $admin): Response
    {
        $form = Form::make('admin.edit')->model($admin)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Users', route('admin.user.index'))
            ->add($admin->full_name, route('admin.user.show', $admin->id))
            ->get();

        return Inertia::render('User/Edit', [
            'form' => $form,
            'breadcrumbs' => $breadcrumbs,
            'title' => "Edit {$admin->full_name}",
        ]);
    }

    public function update(Request $request, Admin $admin): RedirectResponse
    {
        Form::make('admin.edit')
            ->model($admin)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.user.index');
    }

    public function show(Admin $admin): Response
    {
        $form = Form::make('admin.show')->model($admin)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Users', route('admin.user.index'))
            ->add($admin->full_name)
            ->get();

        return Inertia::render('User/Show', [
            'data' => $form,
            'breadcrumbs' => $breadcrumbs,
        ]);
    }

    public function delete(Admin $admin): RedirectResponse
    {
        $admin->user->delete();
        $admin->delete();

        return redirect()->route('admin.user.index');
    }
}
